/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication;

import java.util.logging.Logger;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.microkernel.api.communication.JMXService;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * JMX based remote checker.
 * @author Christophe Hamerling - EBM WebSourcing
 */
@Component(provides = @Interface(name = "service", signature = RemoteCheckerClient.class))
public class JMXRemoteCheckerClientImpl implements RemoteCheckerClient {

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));

    @Requires(name = "jmx")
    protected JMXService jmxService;

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    @Override
    public boolean ping(String containerName) {
        this.log.start();
        
        boolean result = true;
        try {
            // Get a connection to the container for which we check the run state
            final JMXClient server = this.jmxService.getJMXClient(containerName);
            // As a ping is automatically done when getting JMX client, it is not needed to recheck it through a 'ping'
            result = true;
            this.jmxService.returnJMXClient(containerName, server);
        } catch (final Exception e) {
            result = false;
        }

        this.log.end("Result : " + result);
        return result;
    }

}
