/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.deployment;

import java.net.URL;
import java.util.logging.Logger;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.microkernel.api.jbi.management.AtomicDeploymentService;
import org.ow2.petals.microkernel.api.jbi.management.DeploymentServiceMBean;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * This simple deployment service hides all the JMX stuff and the reports
 * handling. When something is OK it returns 'true', if not then 'false' (Waow,
 * this is powerful!!! :D ). It is deicated to be used with only a SA at a time,
 * no more lists...
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 */
@Component(provides = @Interface(name = "service", signature = AtomicDeploymentService.class))
public class AtomicDeploymentServiceImpl implements AtomicDeploymentService {

    @Requires(name = "deployment")
    private DeploymentServiceMBean deploymentService;

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(AtomicDeploymentService.COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    public boolean deploy(URL saURL) throws PetalsException {
        boolean result = true;
        if (saURL == null) {
            throw new PetalsException("Service assembly URL is null");
        }

        try {
            String report = this.deploymentService.deploy(saURL.toString());
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean shutdown(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.shutDown(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean start(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.start(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean stop(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.stop(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }

    public boolean undeploy(String saId) throws PetalsException {
        boolean result = true;
        try {
            String report = this.deploymentService.undeploy(saId);
            // really really bad...
            result = (report.indexOf("Successfully") > -1);

        } catch (Exception e) {
            log.warning(e.getMessage());
            throw new PetalsException(e.getMessage());
        }
        return result;
    }
}
