/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task;

import java.io.File;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.extension.InstallationExtension;
import org.ow2.petals.microkernel.api.extension.InstallationExtensionException;
import org.ow2.petals.microkernel.api.jbi.management.Context;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * This task copies archive file to <i>installed</i> dir.
 * <p>
 * Inputs : archive {@link File} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CopyArchiveToInstalledDirTask extends AbstractFileManipulationTask {

    final List<InstallationExtension> extensions;

    public CopyArchiveToInstalledDirTask(final LoggingUtil log,
            final ContainerConfiguration containerConfiguration,
            final List<InstallationExtension> extensions) {
        super(log, containerConfiguration);
        this.extensions = extensions;
    }

    @Override
    public void execute(Context context) throws Exception {
        File archiveFile = context.getArchiveFile();

        for (final InstallationExtension extension : this.extensions) {
            extension.onInstallation(archiveFile.getAbsolutePath());
        }

        FileUtils.forceDelete(archiveFile);
    }

    @Override
    public void undo(Context context) throws Exception {
        File archiveFile = context.getArchiveFile();

        if (archiveFile != null) {

            for (final InstallationExtension extension : this.extensions) {
                try {
                    extension.onInstallation(archiveFile.getName());
                } catch (final InstallationExtensionException e) {
                    this.log.error("Component archive can not be deleted", e);
                }
            }
        }
    }
}
