/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation;

import java.io.File;
import java.util.List;

import org.ow2.petals.basisapi.exception.PetalsException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class InstallationUtils {

    /**
     * Check the classpath element of the jbi descriptor
     * 
     * @param filePackage
     *            the entity package root
     * @param classPath
     *            a List of classpath elements to check
     * @throws PetalsException
     *             if one of the classpath elements is null, empty or or missing
     */
    public static final void checkClassPathElements(File filePackage, List<String> classPath)
            throws PetalsException {

        for (String pathElement : classPath) {
            // empty class-path elements is not alloweds
            if (StringHelper.isNullOrEmpty(pathElement)) {
                throw new PetalsException("Found empty class-path element in the JBI descriptor");
            }

            // check if the specified path element exists within archive
            if (!".".equals(pathElement) && !hasEntry(filePackage, pathElement)) {
                throw new PetalsException("The class-path element '" + pathElement
                        + "' declared in the " + "JBI descriptor cannot be resolved");
            }
        }
    }

    private static final boolean hasEntry(File filePackage, String pathElement) {
        File artifactFile = new File(filePackage, pathElement);
        return artifactFile.exists();
    }
}
