/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Component.SharedLibrary;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.container.ContainerService;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;

/**
 * This task checks installed shared library for component and register this latter. 
 * <p>
 * A shared librairy must be installed before a component can be
 * installed. Inputs : {@link Jbi}.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public class RegisterComponentInSLTask implements Task {

    private final ContainerService containerService;

    public RegisterComponentInSLTask(final ContainerService containerService) {
        super();
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();

        if (descriptor.getComponent().getSharedLibraryList() != null) {
            for (SharedLibrary sl : descriptor.getComponent().getSharedLibraryList()) {
                SharedLibraryLifeCycle saLC = this.containerService.getSharedLibraryByName(sl.getContent(),
                        sl.getVersion());
                if (saLC == null) {
                    throw new PetalsException("The component depends on the Shared Library '"
                            + sl.getContent() + "' with version '" + sl.getVersion() + "' which is not installed");
                }
                saLC.registerComponent(descriptor.getComponent().getIdentification().getName());
            }
        }
    }

    public void undo(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();

        if (descriptor.getComponent().getSharedLibraryList() != null) {
            for (SharedLibrary sl : descriptor.getComponent().getSharedLibraryList()) {
                SharedLibraryLifeCycle saLC = this.containerService.getSharedLibraryByName(sl.getContent(),
                        sl.getVersion());
                if (saLC != null) {
                    saLC.unregisterComponent(descriptor.getComponent().getIdentification()
                            .getName());
                }
            }
        }

    }

}
