/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module.endpoint;

import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;

/**
 * A handler of a weighted endpoint.
 * @author Roland Naudin - EBM WebSourcing
 */
public class WeightedEndpoint implements Comparable<WeightedEndpoint> {

    /**
     * Localization of the endpoint
     * 
     * @author rnaudin
     * 
     */
    public static enum Localization {
        LOCAL, REMOTE_ACTIVE, REMOTE_INACTIVE
    }

    /**
     * The endpoint instance
     */
    public PetalsServiceEndpoint endpoint;

    /**
     * The weight of the endpoint
     */
    public Float weight;

    /**
     * The localization of the endpoint
     */
    public Localization localization;

    public WeightedEndpoint(final PetalsServiceEndpoint endpoint, final float weighting,
            Localization localization) {
        this.endpoint = endpoint;
        this.weight = new Float(weighting);
        this.localization = localization;
    }

    /**
     * The comparison is reversed
     */
    public int compareTo(final WeightedEndpoint comparatedObject) {
        final int c = this.weight.compareTo(comparatedObject.weight);
        // this is needed because compareTo can return Integer.MIN_VALUE and negating it could give unexpected results
        return c > 0 ? -1 : (c < 0 ? 1 : 0);
    }
}
