/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.server;

import java.util.Properties;

import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.server.exception.PetalsCompositeCtrlException;

/**
 * <p>
 * Controller of the Fractal composite 'Petals'.
 * </p>
 * <p>
 * The Fractal composite 'Petals' is in charge of loading all Fractal architecture. The controller will load all the
 * Fractal architecture except the component 'Configuration' according to the configuration.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface PetalsCompositeController {

    /**
     * Logger name
     */
    public static final String COMPONENT_LOGGER_NAME = "Petals.Server";

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#PETALS_COMPOSITE_CTRL_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Name of the client interface 'configuration' of the composite 'Communication'
     */
    public static final String COMMUNICATION_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'sharedArea' of the composite 'Communication'
     */
    public static final String COMMUNICATION_SHARED_AREA_FRACTAL_ITF_NAME = "sharedArea";

    /**
     * Name of the client interface 'petals-composite-controller' of the composite 'Communication'
     */
    public static final String COMMUNICATION_PETALS_COMPOSITE_CTRL_FRACTAL_ITF_NAME = "petalsCompositeController";

    /**
     * Name of the client interface 'router' of the composite 'Communication'
     */
    public static final String COMMUNICATION_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the server interface 'topology' of the composite 'Communication'
     */
    public static final String COMMUNICATION_TOPOLOGY_FRACTAL_SRV_ITF_NAME = "topology";

    /**
     * Name of the server interface 'jmx' of the composite 'Communication'
     */
    public static final String COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME = "jmx";

    /**
     * Name of the server interface 'jndi' of the composite 'Communication'
     */
    public static final String COMMUNICATION_JNDI_FRACTAL_SRV_ITF_NAME = "jndi";

    /**
     * Name of the client interface 'configuration' of the composite 'System'
     */
    public static final String SYSTEM_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the <b>internal</b> client interface 'configuration' of the composite 'System'
     */
    public static final String SYSTEM_CONFIGURATION_FRACTAL_INT_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the composite 'System'
     */
    public static final String SYSTEM_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the <b>internal</b> client interface 'topology' of the composite 'System'
     */
    public static final String SYSTEM_TOPOLOGY_FRACTAL_INT_ITF_NAME = "topology";

    /**
     * Name of the server interface 'systemstate' of the composite 'System'
     */
    public static final String SYSTEM_SYSTEMSTATE_FRACTAL_SRV_ITF_NAME = "systemState";

    /**
     * Name of the server interface 'sharedArea' of the composite 'System'
     */
    public static final String SYSTEM_SHARED_AREA_FRACTAL_SRV_ITF_NAME = "sharedArea";

    /**
     * Name of the internal server interface 'sharedArea' of the composite 'System'
     */
    public static final String SYSTEM_SHARED_AREA_FRACTAL_INT_SRV_ITF_NAME = "sharedArea";

    /**
     * Name of the server interface 'classloader' of the composite 'System'
     */
    public static final String SYSTEM_CLASSLOADER_FRACTAL_SRV_ITF_NAME = "classloader";

    /**
     * Name of the server interface 'repository' of the composite 'System'
     */
    public static final String SYSTEM_REPOSITORY_FRACTAL_SRV_ITF_NAME = "repository";

    /**
     * Name of the server interface 'service' of the component 'PreExtensionsManager'
     */
    public static final String PREEXTENSIONSMANAGER_FRACTAL_SRV_ITF_NAME = "extensionsManager";

    /**
     * Name of the client interface 'configuration' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'jmx' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'topology' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'systemstate' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'deployment' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_DEPLOYMENT_FRACTAL_ITF_NAME = "deployment";

    /**
     * Name of the client interface 'atomicDeployment' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_ATOMICDEPLOYMENT_FRACTAL_ITF_NAME = "atomicDeployment";

    /**
     * Name of the client interface 'installation' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_INSTALLATION_FRACTAL_ITF_NAME = "installation";

    /**
     * Name of the client interface 'componentInstallation' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_COMPONENTINSTALLATION_FRACTAL_ITF_NAME = "componentInstallation";

    /**
     * Name of the client interface 'endpointDirectory' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_ENDPOINT_DIRECTORY_FRACTAL_ITF_NAME = "endpointDirectory";

    /**
     * Name of the client interface 'admin' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * Name of the client interface 'containerService' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_CONTAINERSERVICE_FRACTAL_ITF_NAME = "containerService";

    /**
     * Name of the client interface 'petalsAdmin' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_PETALSADMIN_FRACTAL_ITF_NAME = "petalsAdmin";

    /**
     * Name of the server interface 'extensionsManager' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_EXTENSIONSMANAGER_FRACTAL_SRV_ITF_NAME = "extensionsManager";

    /**
     * Name of the server interface 'installationExtensionsManager' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_SRV_ITF_NAME = "installationExtensionsManager";

    /**
     * Name of the server interface 'deploymentService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_DEPLOYMENT_FRACTAL_SRV_ITF_NAME = "deploymentService";

    /**
     * Name of the server interface 'atomicDeploymentService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ATOMICDEPLOYMENT_FRACTAL_SRV_ITF_NAME = "atomicDeploymentService";

    /**
     * Name of the server interface 'installationService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_INSTALLATION_FRACTAL_SRV_ITF_NAME = "installationService";

    /**
     * Name of the server interface 'componentInstallationService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_COMPONENTINSTALLATION_FRACTAL_SRV_ITF_NAME = "componentInstallationService";

    /**
     * Name of the server interface 'adminService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ADMIN_FRACTAL_SRV_ITF_NAME = "adminService";

    /**
     * Name of the server interface 'containerService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CONTAINER_FRACTAL_SRV_ITF_NAME = "containerService";

    /**
     * Name of the client interface 'configuration' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'systemstate' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'jmx' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'classloader' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CLASSLOADER_FRACTAL_ITF_NAME = "classloader";

    /**
     * Name of the client interface 'repository' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_REPOSITORY_FRACTAL_ITF_NAME = "repository";

    /**
     * Name of the client interface 'endpointDirectory' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ENDPOINT_DIRECTORY_FRACTAL_ITF_NAME = "endpointDirectory";

    /**
     * Name of the client interface 'sharedArea' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_SHARED_AREA_FRACTAL_ITF_NAME = "sharedArea";

    /**
     * Name of the client interface 'router' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the client interface 'installationExtensionManager' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_ITF_NAME = "installationExtensionManager";

    /**
     * Name of the client interface 'preExtensionsManager' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_PREEXTENSIONSMANAGER_FRACTAL_ITF_NAME = "preExtensionsManager";

    /**
     * Name of the client interface 'postExtensionsManager' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_POSTEXTENSIONSMANAGER_FRACTAL_ITF_NAME = "postExtensionsManager";

    /**
     * Name of the client interface 'jndi' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_JNDI_FRACTAL_ITF_NAME = "jndi";

    /**
     * Name of the server interface 'router' of the composite 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_ROUTER_FRACTAL_SRV_ITF_NAME = "router";

    /**
     * Name of the server interface 'transportlistener' of the composite 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTLISTENER_FRACTAL_SRV_ITF_NAME = "transportlistener";

    /**
     * Name of the server interface 'endpointDirectory' of the composite 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_ENDPOINT_DIRECTORY_FRACTAL_SRV_ITF_NAME = "endpointDirectory";

    /**
     * Name of the client interface 'configuration' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'deployment' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_DEPLOYMENT_FRACTAL_ITF_NAME = "deployment";

    /**
     * Name of the client interface 'installation' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_INSTALLATION_FRACTAL_ITF_NAME = "installation";

    /**
     * Name of the client interface 'configuration' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'jndi' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_JNDI_FRACTAL_ITF_NAME = "jndi";

    /**
     * Name of the client interface 'jmx' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'transporter-local' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTERLOCAL_FRACTAL_ITF_NAME = "transporter-local";

    /**
     * Name of the client interface 'transporter-tcp' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTERTCP_FRACTAL_ITF_NAME = "transporter-tcp";

    /**
     * Name of the client interface 'sharedArea' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_SHARED_AREA_FRACTAL_ITF_NAME = "sharedArea";

    /**
     * Name of the client interface 'sharedArea' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_CONTAINER_FRACTAL_ITF_NAME = "container";

    /**
     * Name of the client interface 'sharedArea' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * Name of the client interface 'configuration' of the component 'Transporter'
     */
    public static final String TRANSPORTER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'transportlistener' of the component 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTLISTENER_FRACTAL_ITF_NAME = "transportlistener";

    /**
     * Name of the client interface 'router' of the component 'Transporter'
     */
    public static final String TRANSPORTER_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the server interface 'transporter-local' of the composite 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTERLOCAL_FRACTAL_SRV_ITF_NAME = "transporter-local";

    /**
     * Name of the server interface 'transporter-tcp' of the composite 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTERTCP_FRACTAL_SRV_ITF_NAME = "transporter-tcp";

    /**
     * Name of the client interface 'configuration' of the component 'Container'
     */
    public static final String CONTAINER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'systemstate' of the component 'Container'
     */
    public static final String CONTAINER_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'classloader' of the composite 'Container'
     */
    public static final String CONTAINER_CLASSLOADER_FRACTAL_ITF_NAME = "classloader";

    /**
     * Name of the client interface 'repository' of the composite 'Container'
     */
    public static final String CONTAINER_REPOSITORY_FRACTAL_ITF_NAME = "repository";

    /**
     * Name of the client interface 'endpointDirectory' of the composite 'Container'
     */
    public static final String CONTAINER_ENDPOINT_DIRECTORY_FRACTAL_ITF_NAME = "endpointDirectory";

    /**
     * Name of the client interface 'router' of the composite 'Container'
     */
    public static final String CONTAINER_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the client interface 'admin' of the composite 'Container'
     */
    public static final String CONTAINER_ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * Stop and shutdown all the Fractal components associated to the JBI components and SAs
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs
     */
    public void stopJbiArtefacts() throws PetalsCompositeCtrlException;

    /**
     * Stop the Fractal composite {@link FractalHelper#JBI_MESSAGING_COMPOSITE}
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping {@link FractalHelper#JBI_MESSAGING_COMPOSITE}
     */
    public void stopJbiMessaging() throws PetalsCompositeCtrlException;

    /**
     * Start the Fractal composite {@link FractalHelper#JBI_MESSAGING_COMPOSITE}
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs starting {@link FractalHelper#JBI_MESSAGING_COMPOSITE}
     */
    public void startJbiMessaging() throws PetalsCompositeCtrlException;

    /**
     * Stop the Fractal component {@link FractalHelper#ENDPOINT_DIRECTORY_COMPONENT}.
     * 
     * @return An {@link AutoCloseable} to restart the component.
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping {@link FractalHelper#ENDPOINT_DIRECTORY_COMPONENT}
     */
    public AutoCloseable pauseEndpointDirectory() throws PetalsCompositeCtrlException;

    /**
     * Start the Fractal composite {@link FractalHelper#TRANSPORTERS_COMPOSITE}
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs starting {@link FractalHelper#TRANSPORTERS_COMPOSITE}
     */
    public void startTransporters() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal sub-system to pause the message exchanges processing:
     * <ol>
     * <li>We wait for all pending exchange finish. If they are not terminated under a given duration, the remaining
     * pending exchanges can be lost or can end with error,</li>
     * <li>and the composite {@link FractalHelper#TRANSPORTERS_COMPOSITE} is stopped to prevent all incoming message
     * exchanges.</li>
     * </ol>
     * </p>
     * 
     * @return An {@link AutoCloseable} to restart the component.
     * @throws PetalsCompositeCtrlException
     *             An error occurs pausing the messaging sub-system
     */
    public AutoCloseable pauseMessagingSubsystem() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal component {@link FractalHelper#SHARED_AREA_COMPONENT}.
     * </p>
     * 
     * @return An {@link AutoCloseable} to restart the component.
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the registry
     */
    public AutoCloseable pauseSharedAreaComponent() throws PetalsCompositeCtrlException;

    /**
     * Retrieve the shared area client implementation of the domain of the current container
     * 
     * @return The shared area client implementation of the domain
     */
    public String getSharedAreaCltImplementation();

    /**
     * <p>
     * Retrieve the configuration of the <b>shared area client</b> mutable implementation as {@link Properties}. See API
     * of the shared area client mutable implementation for the property list.
     * </p>
     * <p>
     * All properties are prefixed by the class name of the shared area client mutable implementation
     * </p>
     * 
     * @param providedTopologyPassphrase
     *            Passphrase needed to return sensible information. If <code>null</code>, empty or not matching the one
     *            of the container on which the connection is established, no sensible information will be returned.
     * @return The configuration of the shared area client mutable implementation as {@link Properties}. Not
     *         <code>null</code>.
     * @throws PetalsCompositeCtrlException
     *             An error occurs getting the configuration of the shared area client mutable implementation
     */
    public Properties getSharedAreaCltConfiguration(final String providedTopologyPassphrase)
            throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal component {@link FractalHelper#SYSTEMRECOVERY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the system recovery service
     */
    public void stopSystemRecoveryService() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Start the Fractal component {@link FractalHelper#SYSTEMRECOVERY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the system recovery service
     */
    public void startSystemRecoveryService() throws PetalsCompositeCtrlException;

    /**
     * The configuration of the Petals container has changed and must be taken into account.
     * 
     * @throws PetalsCompositeCtrlException
     */
    public void onConfigurationUpdated() throws PetalsCompositeCtrlException;

}
