/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.sharedarea;

import java.util.Collection;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.jbi.servicedesc.endpoint.Location;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.implementation.exception.MutableImplException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.api.system.sharedarea.SharedAreaService;
import org.ow2.petals.microkernel.api.system.sharedarea.exception.SharedAreaException;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * A dummy implementation of the shared area just used to simplify the creation of the Fractal architecture by loading
 * an ADL file instead coding all. When the composite {@value FractalHelper#SYSTEM_COMPOSITE} is created, this dummy
 * implementation will be replaced by the expected one.
 * 
 * @author Christophe DENEUX - Linagora
 */
@Component(provides = @Interface(name = SharedAreaService.FRACTAL_SRV_ITF_NAME, signature = SharedAreaService.class))
public class DummySharedAreaImpl implements SharedAreaService {

    protected final LoggingUtil log = new LoggingUtil(Logger.getLogger(COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    @Override
    public Collection<PetalsServiceEndpoint> getEndpoints(EndpointType type) throws SharedAreaException {
        return null;
    }

    @Override
    public Collection<PetalsServiceEndpoint> getEndpointsForInterface(QName interfaceName, EndpointType type)
            throws SharedAreaException {
        return null;
    }

    @Override
    public Collection<PetalsServiceEndpoint> getEndpointsForService(QName serviceName, EndpointType type)
            throws SharedAreaException {
        return null;
    }

    @Override
    public PetalsServiceEndpoint registerInternalEndpoint(String endpointName, QName serviceName, QName[] interfaces,
            Location location, Document description, boolean canOverwrite) throws SharedAreaException {
        return null;
    }

    @Override
    public PetalsServiceEndpoint registerExternalEndpoint(ServiceEndpoint endpoint, Location location,
            Document description, boolean canOverwrite) throws SharedAreaException {
        return null;
    }

    @Override
    public boolean isRegisteredEndpoint(QName serviceName, String endpointName) throws SharedAreaException {
        return false;
    }

    @Override
    public PetalsServiceEndpoint deregisterEndpoint(QName serviceName, String endpointName) throws SharedAreaException {
        return null;
    }

    @Override
    public PetalsServiceEndpoint getEndpoint(QName service, String name) throws SharedAreaException {
        return null;
    }

    @Override
    public Properties getConfiguration(final String providedTopologyPassphrase) {
        return null;
    }

    @Override
    public Document getTopologyPartConfiguration(final Properties properties) throws MutableImplException {
        return null;
    }

    @Override
    public String getImplementationName() {
        return null;
    }

    @Override
    public String getImplementationVersion() {
        return null;
    }

    @Override
    public Document getDescription(ServiceEndpoint endpoint) {
        return null;
    }

    @Override
    public Document getDescription(QName serviceName, String endpointName) throws SharedAreaException {
        return null;
    }

    @Override
    public Collection<SavedEndpoint> leaveTopology() throws SharedAreaException {
        return null;
    }

    @Override
    public void joinTopology(Collection<SavedEndpoint> endpoints) throws SharedAreaException {
        // NOP
    }

    @Override
    public void loadTopology(Collection<SavedEndpoint> endpoints) throws SharedAreaException {
        // NOP
    }
}
