/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.monitoring;

import org.ow2.petals.probes.api.enums.ExecutionStatus;
import org.ow2.petals.probes.api.key.ProbeKey;

/**
 * TCP transporter monitoring service interface.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface TcpTransporterMonitoring extends TcpTransporterMonitoringMBean {

    /**
     * Update the monitoring probe about a new pending outgoing message sent.
     * 
     * @param container
     *            The remote container to which the message is sent.
     */
    public ProbeKey incPendingOutgoingMessagesProbe(final String container);

    /**
     * Update the monitoring probe about an ended outgoing message sent.
     */
    public void endsPendingOutgoingMessagesProbe(final ProbeKey probeKey, final ExecutionStatus status);

    /**
     * Update the monitoring probe about a new pending incoming message sent.
     * 
     * @param container
     *            The remote container from which the message is received.
     */
    public void incPendingIncomingMessagesProbe(final String container);

    /**
     * Update the monitoring probe about an ended incoming message sent.
     * 
     * @param container
     *            The remote container from which the message is received.
     */
    public void endsPendingIncomingMessagesProbe(final String container, final ExecutionStatus status);

    /**
     * A new connection was established to the given remote container.
     */
    public void newOutgoingConnection(final String containerName);

    /**
     * Update the current value of the gauges counting the outgoing connections currently established from the current
     * container to remote containers.
     */
    public void pickOutgoingConnectionNumbers(final String containerName);

    /**
     * A new connection was established from the given remote container.
     */
    public void newIncomingConnection(final String containerName);

    /**
     * Update the current value of the gauges counting the incoming connections currently established from the current
     * container to remote containers.
     */
    public void pickIncomingConnectionNumbers(final String containerIpAddress);

}
