/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.admin;

import java.io.IOException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.rmi.RMIConnectorServer;

import org.ow2.petals.microkernel.communication.jmx.rmi.RMIConnector;

/**
 * Start a JMX server.
 * @author ddesjardins - EBM WebSourcing
 */
public class ServerUtil {

    /**
     * Boolean to know if the server is started
     */
    public static boolean isStarted = false;

    /**
     * JMX connector
     */
    private static JMXConnectorServer cs;

    /**
     * Start the JMX server
     * 
     * @param rmiPort
     *            rmi port of the server
     * @param jndiPort
     *            jndi port of the server
     * @throws Exception
     */
    public static int start(int rmiPort, int jndiPort) throws Exception {
        if (isStarted == false) {
            Registry registry = LocateRegistry.getRegistry(rmiPort);
            try {
                registry.list();
            } catch (Exception e) {
                LocateRegistry.createRegistry(rmiPort);
            }
            MBeanServer beanServer = MBeanServerFactory.newMBeanServer();
            beanServer.registerMBean(new AdminServiceMoc(beanServer), new ObjectName(
                    "FC/Petals/jbi/admin-impl@00001:itf=service"));

            JMXServiceURL jmxUrl = new JMXServiceURL(RMIConnector.getJmxServiceURL("localhost",
                    rmiPort));
            RMIConnectorServer connectorServer = (RMIConnectorServer) JMXConnectorServerFactory
                    .newJMXConnectorServer(jmxUrl, null, beanServer);
            cs = connectorServer;
            cs.start();
            // Start the server
            isStarted = true;
        }
        return rmiPort;
    }

    /**
     * Stop the JMX
     * 
     * @throws IOException
     *             Server
     * 
     */
    public static void stop() throws IOException {
        if (isStarted) {
            isStarted = false;
            cs.stop();
        }
    }
}
