/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.topology;

import java.util.Properties;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.DomainIsNotDynamicException;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.configuration.ConfigurationServiceImpl;
import org.ow2.petals.microkernel.configuration.ConfigurationTestUtils;
import org.ow2.petals.microkernel.configuration.LocalServerPropertiesBuilder;
import org.ow2.petals.topology.generated.DomainMode;
import org.ow2.petals.topology.generated.Topology;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link TopologyServiceImpl#attachContainerTo(ContainerConfiguration, String)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TopologyServiceImpl_DetachContainerTest extends AbstractTopologyServiceImplTest {

    /**
     * Port of the JMX server of the target container
     */
    private final static Integer TARGET_JMX_PORT = Integer.valueOf(7070);

    /**
     * Hostname of the target container
     */
    private final static String TARGET_JMX_HOST = "localhost";

    @Rule
    public final LocalServerPropertiesBuilder localServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    @Rule
    public final LocalServerPropertiesBuilder targetServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    /**
     * <p>
     * Check {@link TopologyServiceImpl#detachContainerTo()} against:
     * <ul>
     * <li>the domain of the current container is static</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs because the domain mode is not <b>dynamic</b>
     * </p>
     */
    @Test(expected = DomainIsNotDynamicException.class)
    public void testCurrentDomainNotDynamic() throws Exception {

        // Create a basic sample topology
        final Topology basicCurrentTopology = ConfigurationTestUtils.createBasicSampleTopology();
        basicCurrentTopology.getDomain().setMode(DomainMode.STATIC);

        // Create and initialize the configuration service
        final Properties currentServerProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);
        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);
        try {
            cfgSvc.loadConfiguration(currentServerProps, basicCurrentTopology);

            final TopologyService topoSvc = new TopologyServiceImpl();
            ReflectionHelper.setPrivateField(topoSvc, "configurationService", cfgSvc);
            this.startTopologyService(topoSvc);

            try {
                topoSvc.detachContainer();
            } finally {
                this.stopTopologyService(topoSvc);
            }
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

}
