/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.Properties;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;

/**
 * Unit tests of {@link ConfigurationServiceImpl#getDomainConfiguration()}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationServiceImpl_GetDomainConfigurationTest extends AbstractConfigurationServiceImplTest {

    @Rule
    public final LocalServerPropertiesBuilder localServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    /**
     * <p>
     * Check {@link ConfigurationServiceImpl#getDomainConfiguration()}
     * </p>
     * <p>
     * Expected result: the {@link DomainConfiguration} returned is correctly initialized
     * </p>
     */
    @Test
    public void testGetDomainConfiguration() throws Exception {

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        try {
            cfgSvc.loadConfiguration(serverProps, this.topology);

            final DomainConfiguration domainConfig = cfgSvc.getDomainConfiguration();
            assertNotNull("No domain configuration found", domainConfig);
            assertEquals("Unexpected domain name", ConfigurationTestUtils.DOMAIN_NAME, domainConfig.getName());
            assertEquals("Unexpected domain description", ConfigurationTestUtils.DOMAIN_DESCRIPTION,
                    domainConfig.getDescription());
            assertEquals("Unexpected domain mode", DomainConfiguration.DomainMode.STATIC, domainConfig.getMode());
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }
}
