/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.extension.utest;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.extension.PetalsPreExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * A Pre-extension controller for unit tests for which dependencies are not satisfied
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DependencyNotSatisfiedPreExtensionController implements PetalsPreExtensionController {

    public final static String ACTIVATION_PROPERTY_NAME = DependencyNotSatisfiedPreExtensionController.class.getName();

    public final static boolean DEFAULT_ACTIVATION = true;

    public final static String EXTENSION_NAME = "Extension name '" + DependencyNotSatisfiedPreExtensionController.class.getName() + "'";

    private final static Class<?> EXTENSION_IMPL = PreExtensionImpl.class;

    @Override
    public Component createFractalComponent(final Component extensionComposite) throws CreationExtensionException {
        try {
            return FractalHelper.createPrimitiveComponent(EXTENSION_IMPL);
        } catch (final InstantiationException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        }
    }

    @Override
    public void removeFractalComponent() {
        // NOP
    }

    @Override
    public String getExtensionName() {
        return EXTENSION_NAME;
    }

    @Override
    public String getExtensionVersion() {
        return null;
    }

    @Override
    public boolean isActivated(final ConfigurationService configurationService) throws PetalsExtensionException {

        return true;
    }

    @Override
    public String[] getDependencies() {
        return new String[] { "a-dependency-not-loaded" };
    }

    @Override
    public boolean failsPetalsStartup() {
        return false;
    }
}
