/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.extension.utest;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.extension.InstallationExtension;
import org.ow2.petals.microkernel.api.extension.PetalsPostExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * A simple post-extension controller providing an installation extension for unit tests
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InstallationExtensionCtrl implements PetalsPostExtensionController {

    public final static String ACTIVATION_PROPERTY_NAME = InstallationExtensionCtrl.class.getName();

    public final static boolean DEFAULT_ACTIVATION = true;

    public final static String EXTENSION_NAME = "Extension name '" + InstallationExtensionCtrl.class.getName() + "'";

    private final static Class<?> EXTENSION_IMPL = InstallationExtensionImpl.class;

    @Override
    public Component createFractalComponent(final Component extensionComposite) throws CreationExtensionException {
        try {
            return FractalHelper.createPrimitiveComponent(EXTENSION_IMPL);
        } catch (final InstantiationException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        }
    }

    @Override
    public void removeFractalComponent() {
        // NOP
    }

    @Override
    public String getExtensionName() {
        return EXTENSION_NAME;
    }

    @Override
    public String getExtensionVersion() {
        return "1.0.0";
    }

    @Override
    public boolean isActivated(final ConfigurationService configurationService) throws PetalsExtensionException {
        final String propertyValue = configurationService.getServerProperties().getProperty(ACTIVATION_PROPERTY_NAME);
        final boolean isActivated;
        if (propertyValue != null) {
            isActivated = Boolean.parseBoolean(propertyValue);
        } else {
            isActivated = DEFAULT_ACTIVATION;
        }

        return isActivated;
    }

    @Override
    public String[] getDependencies() {
        return null;
    }

    @Override
    public boolean failsPetalsStartup() {
        return false;
    }

    /**
     * {@link InstallationExtensionImpl} implements {@link InstallationExtension}.
     */
    @Override
    public boolean isInstallationExtension() {
        return true;
    }

}
