/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.undeploy;

import java.io.FileNotFoundException;
import java.util.logging.Logger;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.util.AbstractMavenTest;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * Test class for the task {@link RemoveSADirectoryTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class RemoveSADirectoryTaskTest extends AbstractMavenTest {

    final static private String SA_NAME = "myServiceAssemblyName";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            RemoveSADirectoryTaskTest.class.getName()));

    /**
     * Check interaction between the tested task and the repository service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSADirectoryTask_000() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        repositorySvc.removeServiceAssembly(SA_NAME);
        EasyMock.replay(repositorySvc);

        final Context ctx = new Context();
        ctx.setEntityName(SA_NAME);
        final Task task = new RemoveSADirectoryTask(log, repositorySvc, null);

        // Process the task.
        task.execute(ctx);

        // Verify interaction between the current test and the repository.
        EasyMock.verify(repositorySvc);
    }

    /**
     * Check interaction between the tested task and the repository service,
     * when processing the task twice for the same service assembly.
     * 
     * @throws Exception
     */
    @Test
    public void testRemoveSADirectoryTask_001() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        repositorySvc.removeServiceAssembly(SA_NAME);
        repositorySvc.removeServiceAssembly(SA_NAME);
        EasyMock.expectLastCall().andThrow(new FileNotFoundException());
        EasyMock.replay(repositorySvc);

        // First, process the task for the 1st installation of the service
        // assembly
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(SA_NAME);
            final Task task1 = new RemoveSADirectoryTask(log, repositorySvc, null);
            task1.execute(ctx1);
        }

        // Second, process the task for the 2nd installation of the service
        // assembly
        {
            final Context ctx2 = new Context();
            ctx2.setEntityName(SA_NAME);
            final Task task2 = new RemoveSADirectoryTask(log, repositorySvc, null);
            task2.execute(ctx2);
            // No exception is raised if an error occurs during task execution.
        }

        // Verify interaction between the current test and the repository
        // service.
        EasyMock.verify(repositorySvc);
    }

}
