/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader.locator;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.junit.Test;

/**
 * Test for the JarFileLocator.
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 */
public class JarFileLocatorTest {

    @Test
    public void testHasFile() throws MalformedURLException, IOException, URISyntaxException {
        final URL jarUrl = Thread.currentThread().getContextClassLoader()
                .getResource("test-classloader-project.jar");
        final JarFileLocator jarFileLocator = new JarFileLocator(jarUrl);
        assertTrue(jarFileLocator.hasFile("META-INF/MANIFEST.MF"));
    }

    @Test
    public void testHasDirectory() throws MalformedURLException, IOException, URISyntaxException {
        final URL jarUrl = Thread.currentThread().getContextClassLoader()
                .getResource("test-classloader-project.jar");
        final JarFileLocator jarFileLocator = new JarFileLocator(jarUrl);
        assertTrue(jarFileLocator.hasDirectory("META-INF"));
    }

    @Test
    public void testListContent() throws MalformedURLException, IOException, URISyntaxException {
        final URL jarUrl = Thread.currentThread().getContextClassLoader()
                .getResource("test-classloader-project.jar");
        final JarFileLocator jarFileLocator = new JarFileLocator(jarUrl);
        assertTrue((jarFileLocator.listContent("META-INF").size() == 1));
    }

    @Test
    public void testHasDirectoryFalse() throws MalformedURLException, IOException,
            URISyntaxException {
        final URL jarUrl = Thread.currentThread().getContextClassLoader()
                .getResource("test-classloader-project.jar");
        final JarFileLocator jarFileLocator = new JarFileLocator(jarUrl);
        assertFalse(jarFileLocator.hasDirectory("foo"));
    }

}
